/**
 * En esta clase se implementa el JDialog que será mostrado cuando se
 * quiera añadir una consulta SPARQL a la aplicación. Se mostrará una
 * ventana donde se puede ver un campo de texto donde insertar el nombre
 * de la consulta y una área de texto donde escribir la consulta
 * 
 * @author Adrián
 */
package UOC.appInterface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Shape;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.EmptyBorder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import UOC.AppPopulate_GTFS_PI.HandleFiles;

import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.RoundRectangle2D;
import java.awt.event.ActionEvent;

public class AddSPARQL extends JDialog {
	private static final Logger log = LogManager.getLogger(AddSPARQL.class);
	private final JPanel contentPanel = new JPanel();
	private JTextField addNombre;
	int x, y;

	/**
	 * Constructor de la clase. Se crea el JDialog para insertar una
	 * consulta SPARQL. Esta ventana consta de un campo de texto para
	 * el nombre y una área de texto para la consulta.
	 * 
	 * @param parent - Contiene el identificador de la ventana principal
	 * @param model - Indicamos si la ventana es modal
	 */
	public AddSPARQL(Frame parent, boolean model) {
		super(parent, model);
		
		// Modificamos la apariencia de la ventana
		this.setUndecorated(true);
		setBackground(new Color(0, 0, 0, 0));
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				// Redondeamos los bordes de la ventana 
				Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 27, 27);
				setShape(forma);
			}
		});
		
		this.setLocationRelativeTo(null);
		
		setBounds(150, 150, 450, 300);
		getContentPane().setLayout(new BorderLayout());
		
		// Creamos el panel que contendrá todos los elementos de la ventana
		JPanel panelVentana = new JPanel();
		panelVentana.setBorder(new EmptyBorder(10, 10, 10, 10));
		getContentPane().add(panelVentana, BorderLayout.CENTER);
		panelVentana.setBackground(new Color(0, 0, 0, 128));
		panelVentana.setLayout(new BorderLayout(0, 0));
		
		/************* contentPanel **************/
		
		// Creamos el panel que contiene los campos de texto
		contentPanel.setBackground(new Color(240,240,240));
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		panelVentana.add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		/* Añadimos los Listener que permiten mover la ventana.
		 * Estos los tenemos que poner tanto en el contentPane1 como en el buttonPane.
		 */
		contentPanel.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		contentPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
		});
		
		// Campo de texto con el nombre de la consulta
		JLabel lblNombreConsulta = new JLabel("Nombre consulta:");
		lblNombreConsulta.setBounds(20, 11, 103, 14);
		contentPanel.add(lblNombreConsulta);
		
		addNombre = new JTextField();
		addNombre.setBounds(20, 28, 390, 20);
		contentPanel.add(addNombre);
		addNombre.setColumns(10);
		
		// Área de texto con la consulta SPARQL
		JLabel lblConsultaSparql = new JLabel("Consulta SPARQL:");
		lblConsultaSparql.setBounds(20, 60, 129, 14);
		contentPanel.add(lblConsultaSparql);
		
		JScrollPane scrollPane1 = new JScrollPane();
		JTextArea addConsulta = new JTextArea();
		scrollPane1.setBounds(20, 84, 390, 153);
		scrollPane1.setViewportView(addConsulta);
		//addConsulta.setBounds(20, 84, 390, 172);
		//contentPanel.add(addConsulta);
		contentPanel.add(scrollPane1);
		
		/*************  buttonPane  **************/
		
		// Añadimos el panel que contiene los botones
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder(new EmptyBorder(5, 5, 5, 15));
		buttonPane.setBackground(new Color(240,240,240));
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		
		// Añadimos los Listener que permiten mover la ventana igual que en contentPane1.
		buttonPane.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		buttonPane.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
		});
		panelVentana.add(buttonPane, BorderLayout.SOUTH);
		
		// Botón de añadir consulta
		JButton okButton = new JButton("A\u00F1adir");
		okButton.setBackground(new Color(240,240,240));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Guardamos la consulta en el archivo y cerramos la ventana
				log.info("Añadiendo consulta SPARQL");
				HandleFiles.guardarConsulta(addNombre.getText(), addConsulta.getText());
				dispose();
			}
		});
		okButton.setActionCommand("OK");
		buttonPane.add(okButton);
		getRootPane().setDefaultButton(okButton);
		
		// Botón de cancelar
		JButton cancelButton = new JButton("Cancelar");
		cancelButton.setBackground(new Color(240,240,240));
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// Si se cancela no se hace ninguna operación y se cierra la ventana
				log.info("Cancelación de añadir consulta SPARQL");
				dispose();
			}
		});
		cancelButton.setActionCommand("Cancel");
		buttonPane.add(cancelButton);
	}
}
