/**
 * En esta clase se implementa el JDialog que será mostrado cuando se
 * quiera eliminar una consulta SPARQL de la aplicación. Se mostrará una
 * ventana donde se puede ver un desplegable, el cual contendrá todas
 * las consultas guardadas previamente, y donde se puede elegir la
 * consulta que se quiere eliminar.
 * 
 * @author Adrián
 */
package UOC.appInterface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Shape;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.RoundRectangle2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import UOC.AppPopulate_GTFS_PI.HandleFiles;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JComboBox;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

public class DeleteSPARQL extends JDialog {

	private static final Logger log = LogManager.getLogger(DeleteSPARQL.class);
	private final JPanel contentPanel = new JPanel();
	int x, y;
	private HashMap<String, String> consultasCombo;

	/**
	 * Constructor de la clase.Constructor de la clase.  Se crea el
	 * JDialog para eliminar una consulta SPARQL. Esta ventana consta
	 * de un JComboBox que contiene las consultas insertadas previamente
	 * en la aplicación.
	 * 
	 * @param parent - Contiene el identificador de la ventana principal
	 * @param model - Indicamos si la ventana es modal 
	 */
	public DeleteSPARQL(Frame parent, boolean model) {
		super(parent, model);
		
		// Modificamos la apariencia de la ventana
		this.setUndecorated(true);
		setBackground(new Color(0, 0, 0, 0));
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				// Redondeamos los bordes de la ventana 
				Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 27, 27);
				setShape(forma);
			}
		});
		
		this.setLocationRelativeTo(null);
		
		setBounds(150, 150, 450, 100);
		getContentPane().setLayout(new BorderLayout());
		
		// Creamos el panel que contendrá todos los elementos de la ventana
		JPanel panelVentana = new JPanel();
		panelVentana.setBorder(new EmptyBorder(10, 10, 10, 10));
		panelVentana.setBackground(new Color(0, 0, 0, 128));
		panelVentana.setLayout(new BorderLayout(0, 0));
		getContentPane().add(panelVentana, BorderLayout.CENTER);
		
		/************* contentPanel **************/
		
		// Creamos el panel que contiene el JComboBox
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPanel.setBackground(new Color(240,240,240));
		
		/* Añadimos los Listener que permiten mover la ventana.
		 * Estos los tenemos que poner tanto en el contentPane1 como en el buttonPane.
		 */
		contentPanel.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		contentPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
		});
		
		panelVentana.add(contentPanel, BorderLayout.CENTER);
		
		
		// Creamos el JComboBox con las consultas añadidas a la aplicación
		JComboBox<String> consultasSPARQL = new JComboBox<String>();
		consultasSPARQL.setBounds(10, 10, 410, 20);
		consultasSPARQL.addItem("--Consultas--");
		consultasSPARQL.addPopupMenuListener(new PopupMenuListener(){

			@Override
			public void popupMenuCanceled(PopupMenuEvent arg0) { }

			@Override
			public void popupMenuWillBecomeInvisible(PopupMenuEvent arg0) {	}

			@Override
			public void popupMenuWillBecomeVisible(PopupMenuEvent arg0) {
				/* Recuperamos las consultas del archivo de consultas y
				 * las vamos añadiendo al desplegable*/
				consultasCombo = HandleFiles.recuperarConsultas();
				DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>)consultasSPARQL.getModel();
				model.removeAllElements();
				model.addElement("--Consultas--");
				for(String key : consultasCombo.keySet()){
					model.addElement(key);
				}
				consultasSPARQL.setModel(model);
			}
			
		});
		contentPanel.setLayout(null);
		contentPanel.add(consultasSPARQL);
		
		/*************  buttonPane  **************/
		
		// Añadimos el panel que contiene los botones
		JPanel buttonPane = new JPanel();
		panelVentana.add(buttonPane, BorderLayout.SOUTH);
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		buttonPane.setBackground(new Color(240,240,240));
		
		// Añadimos los Listener que permiten mover la ventana igual que en contentPane1.
		buttonPane.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		buttonPane.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
		});
		
		// Botón de para eliminar consulta
		JButton deleteButton = new JButton("Eliminar");
		deleteButton.setBackground(new Color(240,240,240));
		deleteButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* Eliminamos la consulta del HashMap y sobrescribimos el archivo de consultas.
				 * Después cerramos la ventana */
				String itemSeleccionado = (String)consultasSPARQL.getSelectedItem();
				if(consultasCombo != null){
					if(consultasCombo.containsKey(itemSeleccionado)){
						log.info("Eliminando consulta SPARQL");
						consultasCombo.remove(itemSeleccionado);
						HandleFiles.guardarConsulta(consultasCombo);
					}
				}
				dispose();
			}
		});
		deleteButton.setActionCommand("OK");
		buttonPane.add(deleteButton);
		getRootPane().setDefaultButton(deleteButton);
		
		// Botón de cancelar
		JButton cancelButton = new JButton("Cancelar");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// Si se cancela no se hace ninguna operación y se cierra la ventana
				log.info("Cancelación de eliminar consulta SPARQL");
				dispose();
			}
		});
		cancelButton.setActionCommand("Cancel");
		cancelButton.setBackground(new Color(240,240,240));
		buttonPane.add(cancelButton);
	}
}
