/**
 * En esta clase se implementa el JFrame principal de la aplicación.
 * En él se mostrara una barra de menú que permitirá realizar diversas
 * acciones, como por ejemplo: mostrar el mapa o ejecutar una consulta.
 * Debajo del menú se mostrara un desplegable con las consultas que se
 * han añadido a la aplicación para poder usarlas de forma rápida.
 * Por último, habrá dos áreas de texto. La de la izquierda es donde
 * estarán las consultas SPARQL que se quieran ejecutar y en la de la
 * derecha se mostrarán los resultados obtenidos.
 * 
 * @author Adrián
 */
package UOC.appInterface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.Shape;
import java.awt.Window;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.RoundRectangle2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import org.apache.jena.query.QuerySolution;
import org.apache.jena.query.ResultSet;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import UOC.AppPopulate_GTFS_PI.HandleFiles;
import UOC.AppPopulate_GTFS_PI.Populate;

import java.awt.Rectangle;
import java.awt.Point;
import javax.swing.JLabel;
import java.awt.Panel;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.awt.Dimension;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import java.awt.Toolkit;

public class MainWindow extends JFrame {

	private static final Logger log = LogManager.getLogger(MainWindow.class);
	private JPanel contentPane;
	static MainWindow frame;
	int x, y;
	private HashMap<String, String> consultasCombo;
	
	/**
	 * Funcion main. En esta función lanzamos la aplicación
	 * para que pueda ser usada por el usuario.
	 * 
	 * @param args - Argumentos pasados para ejecutar la aplicación (No son utilitzados)
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					frame = new MainWindow();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Constructor de la clase. En esta función se crea un JFrame
	 * que contiene un menú con diferentes acciones, un desplegable
	 * con las consultas SPARQL añadidas a la aplicación y unas
	 * áreas de texto donde escribir la consulta y donde se muestran
	 * los resultados.
	 */
	public MainWindow() {
		super("GTFS-PI");
		
		// Cambiamos el icono del JFrame
		setIconImage(Toolkit.getDefaultToolkit().getImage(MainWindow.class.getResource("/resources/uoc2.png")));
		
		// Cargamos la ontología y la poblamos con los datos de los archivos
		Populate m = new Populate();
		m.configuration();
		
		// Modificamos la apariencia de la ventana
		this.setUndecorated(true);
		setBackground(new Color(0, 0, 0, 128));
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				// Redondeamos los bordes de la ventana 
				Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 27, 27);
				setShape(forma);
			}
		});
		
		this.setLocationRelativeTo(null);
		setBounds(100, 100, 450, 300);
		
		// Creamos el panel que contendrá todos los elementos de la ventana
		contentPane = new JPanel();
		contentPane.setOpaque(false);
		contentPane.setBorder(new EmptyBorder(10, 10, 10, 10));
		setContentPane(contentPane);
		contentPane.setLayout(new BorderLayout(0, 0));
		
		/* Creamos la barra de estado de la aplicación.
		 * Esta tendrá los botones de maximizar y cerrar,
		 * un titulo y el icono de la universidad */ 
		JPanel barraEstado = new JPanel();
		barraEstado.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		// Añadimos los Listener que permiten mover la ventana y maximizarla haciendo doble clic
		barraEstado.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		barraEstado.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
			@Override
			public void mouseClicked(MouseEvent evt) {
				// Si se hace doble clic, la ventana se maximiza y se elimina la forma redondeada de los bordes
				if(evt.getClickCount() == 2){
					contentPane.setBorder(new EmptyBorder(0, 0, 0, 0));
					addComponentListener(new ComponentAdapter(){
						public void componentResized(ComponentEvent e){
							Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 0, 0);
							setShape(forma);
						}
					});
					setBounds(0, 0, 450, 300);
					setExtendedState(MAXIMIZED_BOTH);
				}
			}
		});
		/* La barra de estado está dividida en tres columnas, la primera
		 * contiene el icono, la segunda el titulo y la tercera los botones. */
		contentPane.add(barraEstado, BorderLayout.NORTH);
		GridBagLayout gbl_barraEstado = new GridBagLayout();
		gbl_barraEstado.columnWeights = new double[]{0.0, 1.0, 0.0, 0.0};
		gbl_barraEstado.rowWeights = new double[]{0.0};
		barraEstado.setLayout(gbl_barraEstado);
		
		// Creamos el botón X que cierra la aplicación
		JButton exit = new JButton("");
		exit.setBackground(new Color(240,240,240));
		exit.setPreferredSize(new Dimension(39, 23));
		exit.setMinimumSize(new Dimension(39, 23));
		exit.setMaximumSize(new Dimension(39, 23));
		ImageIcon img = new ImageIcon(MainWindow.class.getResource("/resources/Cerrar2.png"));
		exit.setIcon(new ImageIcon(img.getImage().getScaledInstance(25, 17, Image.SCALE_DEFAULT)));
		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.info("Salida de la aplicación");
				// Cerramos la ventana  
				frame.dispose();
				System.exit(0);
			}
		});
		// Al pasar el ratón por encima del botón de cerrar cambiamos la apariencia de este.
		exit.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent arg0) {
				exit.setBackground(new Color(200,200,200));
				ImageIcon img = new ImageIcon(MainWindow.class.getResource("/resources/Cerrar3.png"));
				exit.setIcon(new ImageIcon(img.getImage().getScaledInstance(25, 17, Image.SCALE_DEFAULT)));
			}
			@Override
			public void mouseExited(MouseEvent e) {
				exit.setBackground(new Color(240,240,240));
				ImageIcon img = new ImageIcon(MainWindow.class.getResource("/resources/Cerrar2.png"));
				exit.setIcon(new ImageIcon(img.getImage().getScaledInstance(25, 17, Image.SCALE_DEFAULT)));
			}
		});
		
		// Cargamos el icono que se pondrá en la barra de estado
		JLabel lblIcon = new JLabel("");
		lblIcon.setMinimumSize(new Dimension(30, 15));
		lblIcon.setMaximumSize(new Dimension(30, 15));
		lblIcon.setPreferredSize(new Dimension(30, 15));
		ImageIcon fot = new ImageIcon(MainWindow.class.getResource("/resources/uoc2.png"));
		Icon icono = new ImageIcon(fot.getImage().getScaledInstance(lblIcon.getMinimumSize().width, lblIcon.getMinimumSize().height, Image.SCALE_DEFAULT));
		lblIcon.setIcon(icono);
		GridBagConstraints gbc_lblIcon = new GridBagConstraints();
		gbc_lblIcon.fill = GridBagConstraints.NONE;
		gbc_lblIcon.insets = new Insets(0, 5, 0, 5);
		gbc_lblIcon.gridx = 0;
		gbc_lblIcon.gridy = 0;
		barraEstado.add(lblIcon, gbc_lblIcon);
		
		// Creamos el titulo de la aplicación y que será mostrado en la barra de estado
		JLabel lblTitulo = new JLabel("Ontologia GTFS y Puntos de interés");
		lblTitulo.setHorizontalAlignment(SwingConstants.CENTER);
		GridBagConstraints gbc_lblTitulo = new GridBagConstraints();
		gbc_lblTitulo.fill = GridBagConstraints.BOTH;
		gbc_lblTitulo.insets = new Insets(0, 0, 0, 5);
		gbc_lblTitulo.gridx = 1;
		gbc_lblTitulo.gridy = 0;
		barraEstado.add(lblTitulo, gbc_lblTitulo);
		
		// Creamos el botón que maximiza la ventana
		JButton btnMaxim = new JButton("");
		btnMaxim.setBackground(new Color(240,240,240));
		btnMaxim.setMinimumSize(new Dimension(39, 23));
		btnMaxim.setMaximumSize(new Dimension(39, 23));
		btnMaxim.setPreferredSize(new Dimension(39, 23));
		ImageIcon img2 = new ImageIcon(MainWindow.class.getResource("/resources/maximizar2.png"));
		btnMaxim.setIcon(new ImageIcon(img2.getImage().getScaledInstance(25, 17, Image.SCALE_DEFAULT)));
		
		// Creamos el Listener que permiten maximizar la ventana y que vuelva a su estado original
		btnMaxim.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				/* Si la ventana está maximizada la devolvemos a su
				 * estado original y si está en su estado original
				 * la maximizamos */
				if(getExtendedState() == MAXIMIZED_BOTH){
					contentPane.setBorder(new EmptyBorder(10, 10, 10, 10));
					addComponentListener(new ComponentAdapter(){
						public void componentResized(ComponentEvent e){
							Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 27, 27);
							setShape(forma);
						}
					});
					setExtendedState(NORMAL);
					setBounds(100, 100, 450, 300);
				}else{
					contentPane.setBorder(new EmptyBorder(0, 0, 0, 0));
					addComponentListener(new ComponentAdapter(){
						public void componentResized(ComponentEvent e){
							Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 0, 0);
							setShape(forma);
						}
					});
					setBounds(0, 0, 450, 300);
					setExtendedState(MAXIMIZED_BOTH);
				}
			}
		});
		// Al pasar el ratón por encima del botón de maximizar cambiamos la apariencia de este.
		btnMaxim.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent e) {
				btnMaxim.setBackground(new Color(200,200,200));
			}
			@Override
			public void mouseExited(MouseEvent e) {
				btnMaxim.setBackground(new Color(240,240,240));
			}
		});
		
		GridBagConstraints gbc_btnMaxim = new GridBagConstraints();
		gbc_btnMaxim.insets = new Insets(0, 0, 0, 5);
		gbc_btnMaxim.gridx = 2;
		gbc_btnMaxim.gridy = 0;
		barraEstado.add(btnMaxim, gbc_btnMaxim);
		GridBagConstraints gbc_exit = new GridBagConstraints();
		gbc_exit.fill = GridBagConstraints.NONE;
		gbc_exit.gridx = 3;
		gbc_exit.gridy = 0;
		barraEstado.add(exit, gbc_exit);
		
		// Creamos el panel que contendrá el menú, el desplegable y las áreas de texto
		JPanel ventanaCentral = new JPanel();
		contentPane.add(ventanaCentral, BorderLayout.CENTER);
		ventanaCentral.setLayout(new BorderLayout(0, 0));
		
		// Creamos la barra de menú
		JMenuBar menuBar = new JMenuBar();
		menuBar.setBackground(new Color(200, 200, 200));
		ventanaCentral.add(menuBar, BorderLayout.NORTH);
		
		// Creamos el menú "Archivo" que contiene los ítems de guardar, de salir y de mostrar el mapa
		JMenu mnArchivo = new JMenu("Archivo");
		menuBar.add(mnArchivo);
		
		// Item para guardar la ontologia poblada en un archivo
		JMenuItem mnSave = new JMenuItem("Guardar");
		mnSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.info("Guardando la ontología");
				HandleFiles.salidaArchivo(m.modelofinal);
			}
		});
		
		// Ítem que muestra un  mapa con las posiciones geoespaciales de la ontología
		JMenuItem mnMapa = new JMenuItem("Mapa");
		mnMapa.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				log.info("Mostrando el mapa");
				try{
					// Creamos la consulta SPARQL que pasaremos al mapa
					String consulta = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>"+
							"PREFIX geo: <http://www.w3.org/2003/01/geo/wgs84_pos#>"+
							"PREFIX off: <http://www.GTFS-PI.org/GTFS-PI#>"+

							"SELECT * {"+
							"?place geo:geometry ?geom;"+
							"	off:has_name ?name."+
							"values ?classe {off:Stops off:Interest_point}"+
							"?place  rdf:type ?classe"+
							"}";
					ResultSet results = m.resultadoSPARQL(consulta);
					if(results != null){
						new OsmMapViewer(results).setVisible(true);;
					}
				}catch(Exception exc){
					exc.printStackTrace();
				}
			}
		});
		
		// Ítem para cerrar la aplicación
		JMenuItem mnExit = new JMenuItem("Salir");
		mnExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.info("Cerrando la aplicación");
				frame.dispose();
				System.exit(0);
			}
		});
		mnArchivo.add(mnMapa);
		mnArchivo.add(mnSave);
		mnArchivo.add(mnExit);
		
		// Creamos el menú “SPARQL” que contiene los ítems de añadir y eliminar una consulta
		JMenu mnSparql = new JMenu("SPARQL");
		menuBar.add(mnSparql);
		
		// Ítem que abre el JDialog para añadir una consulta SPARQL
		JMenuItem mnAdd = new JMenuItem("Añadir");
		mnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				log.info("Abriendo dialogo para añadir consulta SPARQL");
				Window ventanaPadre = SwingUtilities.getWindowAncestor((Component)e.getSource());
				new AddSPARQL((Frame)ventanaPadre, true).setVisible(true);
			}
		});
		mnSparql.add(mnAdd);
		
		// Ítem que abre el JDialog para eliminar una consulta SPARQL
		JMenuItem mnDelete = new JMenuItem("Eliminar");
		mnDelete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				log.info("Abriendo dialogo para eliminar consulta SPARQL");
				Window ventanaPadre = SwingUtilities.getWindowAncestor((Component)e.getSource());
				new DeleteSPARQL((Frame)ventanaPadre, true).setVisible(true);
			}
		});
		mnSparql.add(mnDelete);
		
		/* Creamos el menú “Ejecución”, en el tenemos los ítems para ejecutar
		 * una consulta y mostrarla en el área de texto o ejecutarla y mostrarla
		 * en una tabla */
		JMenu mnEjecucin = new JMenu("Ejecución");
		menuBar.add(mnEjecucin);
		
		JMenuItem mnEjecutar = new JMenuItem("Ejecutar");
		mnEjecucin.add(mnEjecutar);
		
		JMenuItem mnTabla = new JMenuItem("Tabla");
		mnEjecucin.add(mnTabla);
		
		// Creamos el panel que contendrá el desplegable y las áreas de texto
		JPanel panelDatos = new JPanel();
		ventanaCentral.add(panelDatos, BorderLayout.CENTER);
		panelDatos.setLayout(new BorderLayout(0, 0));
		
		/* Creamos un JSplitPane que nos permitirá expandir las diferentes áreas de texto
		 * para una utilización más cómoda. Además añadimos las áreas de texto a un
		 * JScrollPane para poder desplazarnos por el texto que desborde el campo. */
		JSplitPane panelCentro = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		panelCentro.setOneTouchExpandable(true);
		panelCentro.setResizeWeight(0.5);
		panelCentro.setBorder(new EmptyBorder(5, 5, 5, 5));
		panelDatos.add(panelCentro, BorderLayout.CENTER);
		
		JScrollPane scrollPane1 = new JScrollPane();
		panelCentro.setLeftComponent(scrollPane1);
		scrollPane1.setBounds(0, 43, 163, 197);
		scrollPane1.setPreferredSize(new Dimension(150, 100));
		
		// Área de texto de las instrucciones SPARQL
		JTextArea instrucciones = new JTextArea();
		scrollPane1.setViewportView(instrucciones);
		
		JScrollPane scrollPane2 = new JScrollPane();
		panelCentro.setRightComponent(scrollPane2);
		scrollPane2.setBounds(0, 43, 163, 197);
		scrollPane2.setPreferredSize(new Dimension(150, 100));
		
		// Área de texto para los resultados de la consulta
		JTextArea ejecucion = new JTextArea();
		scrollPane2.setViewportView(ejecucion);
		ejecucion.setBounds(271, 43, 163, 197);
		
		// Creamos el panel que contendrá el desplegable con las consultas añadidas a la aplicación
		JPanel panelNorte = new JPanel();
		panelDatos.add(panelNorte, BorderLayout.NORTH);
		panelNorte.setLayout(new BorderLayout(0, 0));
		panelNorte.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		JComboBox<String> consultasSPARQL = new JComboBox();
		panelNorte.add(consultasSPARQL, BorderLayout.NORTH);
		consultasSPARQL.addItem("--Consultas--");
		
		consultasSPARQL.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				// Cuando cambiamos de selección actualizamos el área de texto de las instrucciones
				String itemSeleccionado = (String)consultasSPARQL.getSelectedItem();
				if(consultasCombo.containsKey(itemSeleccionado))
					instrucciones.setText(consultasCombo.get(itemSeleccionado).replace("\\u000A", "\n").replace("%2C", ","));
			}
		});
		
		consultasSPARQL.addPopupMenuListener(new PopupMenuListener(){
			@Override
			public void popupMenuCanceled(PopupMenuEvent arg0) { }

			@Override
			public void popupMenuWillBecomeInvisible(PopupMenuEvent arg0) {	}

			@Override
			public void popupMenuWillBecomeVisible(PopupMenuEvent arg0) {
				// Cuando desplegamos el JComboBox cargamos las consultas del archivo de consultas
				consultasCombo = HandleFiles.recuperarConsultas();
				DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>)consultasSPARQL.getModel();
				model.removeAllElements();
				model.addElement("--Consultas--");
				for(String key : consultasCombo.keySet()){
					model.addElement(key);
				}
				consultasSPARQL.setModel(model);
			}
		});
		consultasSPARQL.setBounds(10, 11, 414, 20);
		
		// Ítem que ejecuta una consulta SPARQL y muestra el resultado en el área de texto de resultados
		mnEjecutar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.info("Ejecución directa de consulta SPARQL");
				ResultSet results;
				try {
					results = m.resultadoSPARQL(instrucciones.getText());
					if(results != null){
						// Se limpia el área de texto
						ejecucion.setText(null);
						for ( ; results.hasNext() ; ) {
							QuerySolution soln = results.nextSolution() ;
							//log.info("-> "+soln);
						    ejecucion.setText(ejecucion.getText() + soln.toString()+"\n");
						}
					}
				} catch (Exception e) {
					log.error("Error al ejecutar la consulta SPARQL.");
					//e.printStackTrace();
				}
			}
		});
		/* Ítem que ejecuta una consulta SPARQL y muestra el resultado en una tabla.
		 * La tabla estará en un jDialog para mejorar la visualización  */
		mnTabla.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				log.info("Ejecución en una tabla de consulta SPARQL");
				try {
					// Si la consulta es diferente de null abrimos el JDialog y mostramos los resultados
					Window ventanaPadre = SwingUtilities.getWindowAncestor((Component)e.getSource());
					ResultSet results = m.resultadoSPARQL(instrucciones.getText());
					if(results != null){
						new SolutionTable((Frame)ventanaPadre, true, results).setVisible(true);
					}
				} catch (Exception exc) {
					log.error("Error al ejecutar la consulta SPARQL y mostrarla en una tabla.");
					//exc.printStackTrace();
				}
			}
		});
	}
}
